@@ -34,6 +34,7 @@ FROM = keyword("from").suppress()
 FULL = keyword("full")
 GROUP = keyword("group").suppress()
 HAVING = keyword("having").suppress()
+ILIKE = keyword("ilike")
 INNER = keyword("inner")
 INTERVAL = keyword("interval")
 JOIN = keyword("join")
@@ -48,6 +49,7 @@ OUTER = keyword("outer")
 OVER = keyword("over").suppress()
 PARTITION = keyword("partition").suppress()
 # PERCENT = keyword("percent").suppress()
+QUALIFY = keyword("qualify").suppress()
 RIGHT = keyword("right")
 RLIKE = keyword("rlike")
 SAFE_CAST = keyword("safe_cast")
@@ -129,6 +131,8 @@ IS = keyword("is")
 NOT = keyword("not")
 OR = keyword("or")
 LATERAL = keyword("lateral")
+PIVOT = keyword("pivot")
+UNPIVOT = keyword("unpivot")
 VIEW = keyword("view")
 
 # COMPOUND KEYWORDS
@@ -156,6 +160,7 @@ ORDER_BY = Group(ORDER + BY).set_parser_name("order by")
 # COMPOUND OPERATORS
 AT_TIME_ZONE = Group(keyword("at") + keyword("time") + keyword("zone"))
 NOT_BETWEEN = Group(NOT + BETWEEN).set_parser_name("not_between")
+NOT_ILIKE = Group(NOT + ILIKE).set_parser_name("not_ilike")
 NOT_LIKE = Group(NOT + LIKE).set_parser_name("not_like")
 NOT_RLIKE = Group(NOT + RLIKE).set_parser_name("not_rlike")
 NOT_IN = Group(NOT + IN).set_parser_name("nin")
@@ -213,7 +218,9 @@ RESERVED = MatchFirst([
     OUTER,
     OVER,
     PARTITION,
+    PIVOT,
     PRIMARY,
+    QUALIFY,
     REFERENCES,
     RIGHT,
     RLIKE,
@@ -224,6 +231,7 @@ RESERVED = MatchFirst([
     UNION,
     UNIQUE,
     UNNEST,
+    UNPIVOT,
     USING,
     WHEN,
     WHERE,
@@ -323,8 +331,8 @@ KNOWN_OPS = [
     NOT_IN,
     IS_NOT,
     IS,
-    LIKE,
-    NOT_LIKE,
+    LIKE, ILIKE,
+    NOT_LIKE, NOT_ILIKE,
     RLIKE,
     NOT_RLIKE,
     SIMILAR_TO,
