@@ -29,48 +29,36 @@ def no_dashes(tokens, start, string):
 
 digit = Char("0123456789")
 with whitespaces.NO_WHITESPACE:
-    simple_ident = (
+    ident_w_dash = (
         Char(FIRST_IDENT_CHAR)
         + (Regex("(?<=[^ 0-9])\\-(?=[^ 0-9])") | Char(IDENT_CHAR))[...]
     )
-    simple_ident = Regex(simple_ident.__regex__()[1]) / no_dashes
+    ident_w_dash = Regex(ident_w_dash.__regex__()[1]) / no_dashes
+
+simple_ident = Word(FIRST_IDENT_CHAR, IDENT_CHAR)
 
 
 def common_parser():
-    combined_ident = Combine(delimited_list(
-        ansi_ident | mysql_backtick_ident | simple_ident, separator=".", combine=True,
-    )).set_parser_name("identifier")
-
-    return parser(regex_string | ansi_string, combined_ident)
+    atomic_ident = ansi_ident | mysql_backtick_ident | simple_ident
+    return parser(regex_string | ansi_string, atomic_ident)
 
 
 def mysql_parser():
     utils.emit_warning_for_double_quotes = False
 
     mysql_string = regex_string | ansi_string | mysql_doublequote_string
-    mysql_ident = Combine(delimited_list(
-        mysql_backtick_ident | sqlserver_ident | simple_ident,
-        separator=".",
-        combine=True,
-    )).set_parser_name("mysql identifier")
-
-    return parser(mysql_string, mysql_ident)
+    atomic_ident = mysql_backtick_ident | sqlserver_ident | ident_w_dash
+    return parser(mysql_string, atomic_ident)
 
 
 def sqlserver_parser():
-    combined_ident = Combine(delimited_list(
-        ansi_ident
-        | mysql_backtick_ident
-        | sqlserver_ident
-        | Word(FIRST_IDENT_CHAR, IDENT_CHAR),
-        separator=".",
-        combine=True,
-    )).set_parser_name("identifier")
-
-    return parser(regex_string | ansi_string, combined_ident, sqlserver=True)
+    atomic_ident = ansi_ident | mysql_backtick_ident | sqlserver_ident | simple_ident
+    return parser(regex_string | ansi_string, atomic_ident, sqlserver=True)
 
 
-def parser(literal_string, ident, sqlserver=False):
+def parser(literal_string, simple_ident, sqlserver=False):
+    ident = Combine(delimited_list(simple_ident, separator=".", combine=True))
+
     with Whitespace() as white:
         rest_of_line = Regex(r"[^\n]*")
 
@@ -282,6 +270,16 @@ def parser(literal_string, ident, sqlserver=False):
             + RB
         ) / to_json_call
 
+        dynamic_accessor = (
+            Literal("[").suppress() + expression + Literal("]").suppress()
+        )
+        simple_accessor = Literal(".").suppress() + simple_ident / to_literal
+        accessor = (
+            Literal(":").suppress()
+            + Group(simple_ident / to_literal | dynamic_accessor)
+            + ZeroOrMore(Group(simple_accessor | dynamic_accessor))
+        )
+
         with NO_WHITESPACE:
 
             def scale(tokens):
@@ -328,20 +326,9 @@ def parser(literal_string, ident, sqlserver=False):
                 | infix_notation(
                     compound,
                     [
-                        (
-                            Literal("[").suppress()
-                            + expression
-                            + Literal("]").suppress(),
-                            1,
-                            LEFT_ASSOC,
-                            to_offset,
-                        ),
-                        (
-                            Literal(".").suppress() + simple_ident,
-                            1,
-                            LEFT_ASSOC,
-                            to_offset,
-                        ),
+                        (dynamic_accessor, 1, LEFT_ASSOC, to_offset,),
+                        (simple_accessor, 1, LEFT_ASSOC, to_offset,),
+                        (accessor, 1, LEFT_ASSOC, to_offset),
                         (window_clause, 1, LEFT_ASSOC, to_window_mod),
                         (
                             assign("filter", LB + WHERE + expression + RB),
@@ -369,15 +356,43 @@ def parser(literal_string, ident, sqlserver=False):
 
         table_source = Forward()
 
+        pivot_join = (
+            PIVOT("op")
+            + (
+                LB
+                + expression("aggregate")
+                + assign("for", identifier)
+                + (IN + expression("in"))
+                + RB
+                + alias
+            )("kwargs")
+        ) / to_pivot_call
+
+        unpivot_join = (
+            UNPIVOT("op")
+            + (
+                LB
+                + (expression("value") + assign("for", identifier) + IN)
+                + (LB + delimited_list(expression)("in") + RB)
+                + RB
+                + alias
+            )("kwargs")
+        ) / to_unpivot_call
+
         join = (
-            Group(joins)("op")
-            + table_source("join")
-            + Optional((ON + expression("on")) | (USING + expression("using")))
+            pivot_join
+            | unpivot_join
             | (
-                Group(WINDOW)("op")
-                + Group(identifier("name") + AS + over_clause("value"))("join")
+                Group(joins)("op")
+                + table_source("join")
+                + Optional((ON + expression("on")) | (USING + expression("using")))
+                | (
+                    Group(WINDOW)("op")
+                    + Group(identifier("name") + AS + over_clause("value"))("join")
+                )
             )
-        ) / to_join_call
+            / to_join_call
+        )
 
         selection = (
             (
@@ -412,6 +427,7 @@ def parser(literal_string, ident, sqlserver=False):
                 + Optional(WHERE + expression("where"))
                 + Optional(GROUP_BY + delimited_list(Group(named_column))("groupby"))
                 + Optional(HAVING + expression("having"))
+                + Optional(QUALIFY + expression("qualify"))
             )
         )
 
@@ -459,44 +475,23 @@ def parser(literal_string, ident, sqlserver=False):
             + Optional(assign("repeatable", LB + int_num + RB))
         )("tablesample")
 
-        value_column = (
-            TRUE | FALSE | timestamp | literal_string | hex_num | real_num | int_num
-        )
-
-        pivot_table = assign(
-            "pivot",
-            LB
-            + (expression("aggregate") + assign("for", identifier) + IN)
-            + (LB + delimited_list(value_column)("in") + RB)
-            + RB,
-        )
-
         unnest = (UNNEST("op") + LB + expression("params") + RB) / to_json_call
-
-        # <pivoted_table> ::=
-        #     table_source PIVOT <pivot_clause> [ [ AS ] table_alias ]
-        #
-        # <pivot_clause> ::=
-        #         ( aggregate_function ( value_column [ [ , ]...n ])
-        #         FOR pivot_column
-        #         IN ( <column_list> )
-        #     )
-        #
-        # <unpivoted_table> ::=
-        #     table_source UNPIVOT <unpivot_clause> [ [ AS ] table_alias ]
-        #
-        # <unpivot_clause> ::=
-        #     ( value_column FOR pivot_column IN ( <column_list> ) )
-        lateral_source = (LATERAL("op")+table_source("params"))/to_json_call
+        lateral_source = (LATERAL("op") + table_source("params")) / to_json_call
 
         table_source << Group(
-            (lateral_source | (LB + query + RB) | unnest | stack | call_function | identifier)("value")
+            (
+                lateral_source
+                | (LB + query + RB)
+                | unnest
+                | stack
+                | call_function
+                | identifier
+            )("value")
             + MatchAll([
                 Optional(flag("with ordinality")),
                 Optional(WITH + LB + keyword("nolock")("hint") + RB),
                 Optional(WITH + OFFSET + Optional(AS) + identifier("with_offset")),
                 Optional(tablesample),
-                Optional(pivot_table),
                 alias,
             ])
         ) / to_table
@@ -686,7 +681,8 @@ def parser(literal_string, ident, sqlserver=False):
             keyword("update")("op")
             + identifier("params")
             + assign("set", Dict(delimited_list(Group(identifier + EQ + expression))))
-            + Optional(assign("where", expression))
+            + Optional((FROM + delimited_list(table_source) + ZeroOrMore(join))("from"))
+            + Optional(WHERE + expression("where"))
             + returning
         ) / to_json_call
 
@@ -698,6 +694,65 @@ def parser(literal_string, ident, sqlserver=False):
             + returning
         ) / to_json_call
 
+        #############################################################
+        # PROCEEDURAL
+        #############################################################
+        set = (
+            keyword("set")("op")
+            + (identifier + EQ + expression)("params") / (lambda t: {t[0]: t[1]})
+        ) / to_json_call
+
+        unset = (keyword("unset")("op") + identifier("params")) / to_json_call
+
+        copy_options = Forward()
+        copy_options << ZeroOrMore(MatchFirst(
+            [
+                keyword(n).suppress()
+                + EQ
+                + (LB + copy_options + RB | expression)(n.lower())
+                for n in copy_params
+            ]
+            + [PARTITION_BY.suppress() + expression("partition_by")]
+        ))
+
+        with NO_WHITESPACE:
+            file_name = Regex("[a-zA-Z0-9-_!.]+")
+            file_path = Optional(
+                "/"
+                + delimited_list(file_name, separator="/", combine=True)
+                + Optional("/")
+            )
+            # @%load1/data1/
+            file_source = Combine(
+                Literal("@")
+                + (
+                    Literal("~") + file_path
+                    | Literal("%") + file_name + file_path
+                    | (
+                        simple_ident
+                        + Optional("." + Optional("%") + file_name)
+                        + file_path
+                    )
+                )
+                | (
+                    CaselessLiteral("azure")
+                    | CaselessLiteral("s3")
+                    | CaselessLiteral("gcs")
+                )
+                + "://"
+                + file_name
+                + file_path
+            )
+
+        copy = assign(
+            "copy",
+            (
+                assign("into", file_source | expression)
+                + Optional(assign("from", file_source | expression))
+                + copy_options
+            ),
+        )
+
         set_parser_names()
 
         return (
@@ -705,4 +760,6 @@ def parser(literal_string, ident, sqlserver=False):
             | (insert | update | delete)
             | (create_table | create_view | create_cache | create_index)
             | (drop_table | drop_view | drop_index)
+            | copy
+            | (Optional(keyword("alter session")).suppress() + (set | unset))
         ).finalize()
