@@ -10,7 +10,7 @@
 import ast
 import sys
 
-from mo_dots import is_data, is_null, Data, from_data, literal_field
+from mo_dots import is_data, is_null, Data, from_data, literal_field, unliteral_field
 from mo_future import text, number_types, binary_type, flatten
 from mo_imports import expect
 from mo_parsing import *
@@ -193,7 +193,7 @@ def to_json_operator(tokens):
     operands = [tokens[0], tokens[2]]
     binary_op = Call(op, operands, {})
 
-    if op in {"add", "mul", "and", "or"}:
+    if op in {"add", "mul", "and", "or", "concat", "binary_and", "binary_or"}:
         # ASSOCIATIVE OPERATORS
         acc = []
         for operand in operands:
@@ -219,7 +219,7 @@ def to_json_operator(tokens):
 
 def to_offset(tokens):
     expr, offset = tokens.tokens
-    return Call("get", [expr, offset], {})
+    return Call("get", [expr, *offset], {})
 
 
 def to_window_mod(tokens):
@@ -247,6 +247,7 @@ def to_tuple_call(tokens):
 binary_ops = {
     "::": "cast",
     "COLLATE": "collate",
+    ":": "get",
     "||": "concat",
     "*": "mul",
     "/": "div",
@@ -306,7 +307,11 @@ def to_trim_call(tokens):
 
 
 def to_kwarg(tokens):
-    return {k: v for k,v in [tuple(tokens)]}
+    return {k: v for k, v in [tuple(tokens)]}
+
+
+def to_literal(t):
+    return {"literal": unliteral_field(t[0])}
 
 
 def to_json_call(tokens):
@@ -385,6 +390,14 @@ def to_join_call(tokens):
     return output
 
 
+def to_pivot_call(tokens):
+    return {"pivot": dict(tokens["kwargs"])}
+
+
+def to_unpivot_call(tokens):
+    return {"unpivot": dict(tokens["kwargs"])}
+
+
 def to_expression_call(tokens):
     if set(tokens.keys()) & {"over", "within", "filter"}:
         return
@@ -580,7 +593,7 @@ def to_table(tokens):
 
 def single_literal(tokens):
     val = tokens[0]
-    val = '"""' + val[1:-1].replace("''", "\\'") + '"""'
+    val = '"""' + val[1:-1].replace("''", "\\'").replace('"', '\\"') + '"""'
     return {"literal": ast.literal_eval(val)}
 
 
@@ -671,3 +684,62 @@ mysql_doublequote_string = Regex(r'\"(\"\"|[^"])*\"') / double_literal
 ansi_ident = Regex(r'\"(\"\"|[^"])*\"') / double_column
 mysql_backtick_ident = Regex(r"\`(\`\`|[^`])*\`") / backtick_column
 sqlserver_ident = Regex(r"\[(\]\]|[^\]])*\]") / square_column
+
+copy_params = (
+    "ALLOW_DUPLICATE",
+    "AWS_KEY_ID",
+    "AWS_SECRET_KEY",
+    "AWS_TOKEN",
+    "AZURE_SAS_TOKEN",
+    "BINARY_AS_TEXT",
+    "BINARY_FORMAT",
+    "COMPRESSION",
+    "CREDENTIALS",
+    "DATE_FORMAT",
+    "DISABLE_AUTO_CONVERT",
+    "DISABLE_SNOWFLAKE_DATA",
+    "EMPTY_FIELD_AS_NULL",
+    "ENABLE_OCTA",
+    "ENCODING",
+    "ENCRYPTION",
+    "ENFORCE_LENGTH",
+    "ERROR_ON_COLUMN_COUNT_MISMATCH",
+    "ESCAPE_UNENCLOSED_FIELD",
+    "ESCAPE",
+    "FIELD_DELIMITER",
+    "FIELD_OPTIONALLY_ENCLOSED_BY",
+    "FILES",
+    "FILE_FORMAT",
+    "FORCE",
+    "FORMAT_NAME",
+    "HEADER",
+    "IGNORE_UTF8_ERRORS",
+    "INCLUDE_QUERY_ID",
+    "LOAD_UNCERTAIN_FILES",
+    "MASTER_KEY",
+    "MATCH_BY_COLUMN_NAME",
+    "MAX_FILE_SIZE",
+    "NULL_IF",
+    "ON_ERROR",
+    "PATTERN",
+    "PRESERVE_SPACE",
+    "PURGE",
+    "RECORD_DELIMITER",
+    "REPLACE_INVALID_CHARACTERS",
+    "RETURN_FAILED_ONLY",
+    "SINGLE",
+    "SIZE_LIMIT",
+    "SKIP_BLANK_LINES",
+    "SKIP_BYTE_ORDER_MARK",
+    "SKIP_HEADER",
+    "STORAGE_INTEGRATION",
+    "STRIP_NULL_VALUES",
+    "STRIP_OUTER_ARRAY",
+    "STRIP_OUTER_ELEMENT",
+    "TIME_FORMAT",
+    "TIMESTAMP_FORMAT",
+    "TRIM_SPACE",
+    "TYPE",
+    "TRUNCATECOLUMNS",
+    "VALIDATION_MODE",
+)
